@@ -48,6 +48,16 @@ const char *cuewCompilerPath();
 int cuewCompilerVersion();
 #  endif /* WITH_CUDA_DYNLOAD */
 
+static inline bool cudaSupportsDevice(const int cudaDevID)
+{
+  int major;
+  cuDeviceGetAttribute(&major, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MAJOR, cudaDevID);
+  if (major >= 3) {
+    return true;
+  }
+  return false;
+}
+
 CCL_NAMESPACE_END
 
 #endif /* WITH_CUDA */
