@@ -126,8 +126,7 @@ static void subdiv_mesh_prepare_accumulator(SubdivMeshContext *ctx, int num_vert
   if (!ctx->have_displacement) {
     return;
   }
-  ctx->accumulated_counters = static_cast<int *>(
-      MEM_calloc_arrayN(num_vertices, sizeof(*ctx->accumulated_counters), __func__));
+  ctx->accumulated_counters = MEM_calloc_arrayN<int>(num_vertices, __func__);
 }
 
 static void subdiv_mesh_context_free(SubdivMeshContext *ctx)
@@ -135,7 +134,7 @@ static void subdiv_mesh_context_free(SubdivMeshContext *ctx)
   MEM_SAFE_FREE(ctx->accumulated_counters);
   MEM_SAFE_FREE(ctx->subdiv_corner_verts);
   MEM_SAFE_FREE(ctx->subdiv_corner_edges);
-  CustomData_free(&ctx->coarse_corner_data_interp, ctx->coarse_mesh->corners_num);
+  CustomData_free(&ctx->coarse_corner_data_interp);
 }
 
 /** \} */
@@ -307,7 +306,7 @@ static void vertex_interpolation_from_corner(const SubdivMeshContext *ctx,
 static void vertex_interpolation_end(VerticesForInterpolation *vertex_interpolation)
 {
   if (vertex_interpolation->vertex_data_storage_allocated) {
-    CustomData_free(&vertex_interpolation->vertex_data_storage, 4);
+    CustomData_free(&vertex_interpolation->vertex_data_storage);
   }
 }
 
@@ -431,7 +430,7 @@ static void loop_interpolation_from_corner(const SubdivMeshContext *ctx,
 static void loop_interpolation_end(LoopsForInterpolation *loop_interpolation)
 {
   if (loop_interpolation->corner_data_storage_allocated) {
-    CustomData_free(&loop_interpolation->corner_data_storage, 4);
+    CustomData_free(&loop_interpolation->corner_data_storage);
   }
 }
 
@@ -545,13 +544,13 @@ static bool subdiv_mesh_topology_info(const ForeachContext *foreach_context,
   Mesh &subdiv_mesh = *subdiv_context->subdiv_mesh;
   BKE_mesh_copy_parameters_for_eval(subdiv_context->subdiv_mesh, &coarse_mesh);
 
-  CustomData_free(&subdiv_mesh.vert_data, 0);
+  CustomData_free(&subdiv_mesh.vert_data);
   CustomData_init_layout_from(
       &coarse_mesh.vert_data, &subdiv_mesh.vert_data, mask.vmask, CD_SET_DEFAULT, num_vertices);
-  CustomData_free(&subdiv_mesh.edge_data, 0);
+  CustomData_free(&subdiv_mesh.edge_data);
   CustomData_init_layout_from(
       &coarse_mesh.edge_data, &subdiv_mesh.edge_data, mask.emask, CD_SET_DEFAULT, num_edges);
-  CustomData_free(&subdiv_mesh.face_data, 0);
+  CustomData_free(&subdiv_mesh.face_data);
   CustomData_init_layout_from(
       &coarse_mesh.face_data, &subdiv_mesh.face_data, mask.pmask, CD_SET_DEFAULT, num_faces);
   if (num_faces != 0) {
@@ -563,11 +562,9 @@ static bool subdiv_mesh_topology_info(const ForeachContext *foreach_context,
                        &subdiv_context->coarse_corner_data_interp,
                        mask.lmask,
                        coarse_mesh.corners_num);
-  CustomData_free_layer_named(
-      &subdiv_context->coarse_corner_data_interp, ".corner_vert", coarse_mesh.corners_num);
-  CustomData_free_layer_named(
-      &subdiv_context->coarse_corner_data_interp, ".corner_edge", coarse_mesh.corners_num);
-  CustomData_free(&subdiv_mesh.corner_data, 0);
+  CustomData_free_layer_named(&subdiv_context->coarse_corner_data_interp, ".corner_vert");
+  CustomData_free_layer_named(&subdiv_context->coarse_corner_data_interp, ".corner_edge");
+  CustomData_free(&subdiv_mesh.corner_data);
   CustomData_init_layout_from(&subdiv_context->coarse_corner_data_interp,
                               &subdiv_mesh.corner_data,
                               mask.lmask,
@@ -575,10 +572,8 @@ static bool subdiv_mesh_topology_info(const ForeachContext *foreach_context,
                               num_loops);
 
   /* Allocate corner topology arrays which are added to the result at the end. */
-  subdiv_context->subdiv_corner_verts = static_cast<int *>(
-      MEM_malloc_arrayN(num_loops, sizeof(int), __func__));
-  subdiv_context->subdiv_corner_edges = static_cast<int *>(
-      MEM_malloc_arrayN(num_loops, sizeof(int), __func__));
+  subdiv_context->subdiv_corner_verts = MEM_malloc_arrayN<int>(size_t(num_loops), __func__);
+  subdiv_context->subdiv_corner_edges = MEM_malloc_arrayN<int>(size_t(num_loops), __func__);
 
   subdiv_mesh_ctx_cache_custom_data_layers(subdiv_context);
   subdiv_mesh_prepare_accumulator(subdiv_context, num_vertices);
