@@ -521,8 +521,7 @@ TEST_F(keyframes_paste, pastebuf_match_path_property)
 
     bArmature *armature = BKE_armature_add(bmain, "Armature");
     for (const auto &bone_name : {"hand.L", "hand.R", "middle"}) {
-      Bone *bone = static_cast<Bone *>(MEM_mallocN(sizeof(Bone), __func__));
-      memset(bone, 0, sizeof(Bone));
+      Bone *bone = MEM_callocN<Bone>(__func__);
       STRNCPY(bone->name, bone_name);
       BLI_addtail(&armature->bonebase, bone);
     }
