@@ -8,14 +8,12 @@
 
 #pragma once
 
-#include <stdio.h>
+#include <cstdio>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+struct ImFileColorSpace;
 
-void imb_initopenexr(void);
-void imb_exitopenexr(void);
+void imb_initopenexr();
+void imb_exitopenexr();
 
 /**
  * Test presence of OpenEXR file.
@@ -25,15 +23,14 @@ bool imb_is_a_openexr(const unsigned char *mem, size_t size);
 
 bool imb_save_openexr(struct ImBuf *ibuf, const char *filepath, int flags);
 
-struct ImBuf *imb_load_openexr(const unsigned char *mem, size_t size, int flags, char *colorspace);
+struct ImBuf *imb_load_openexr(const unsigned char *mem,
+                               size_t size,
+                               int flags,
+                               ImFileColorSpace &r_colorspace);
 
 struct ImBuf *imb_load_filepath_thumbnail_openexr(const char *filepath,
                                                   int flags,
                                                   size_t max_thumb_size,
-                                                  char colorspace[],
+                                                  ImFileColorSpace &r_colorspace,
                                                   size_t *r_width,
                                                   size_t *r_height);
-
-#ifdef __cplusplus
-}
-#endif
